@@ -1,7 +1,7 @@
 # -*- coding:utf-8 -*-
 ## src/common/helpers.py
 ##
-## Copyright (C) 2003-2010 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2003-2012 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2005-2006 Dimitur Kirov <dkirov AT gmail.com>
 ##                         Nikos Kouremenos <kourem AT gmail.com>
 ## Copyright (C) 2006 Alex Mauer <hawke AT hawkesnest.net>
@@ -35,11 +35,16 @@ import locale
 import os
 import subprocess
 import urllib
+import urllib2
+import webbrowser
 import errno
 import select
 import base64
 import hashlib
+import shlex
 import caps_cache
+import socket
+import time
 
 from encodings.punycode import punycode_encode
 from string import Template
@@ -51,9 +56,14 @@ try:
     import winsound # windows-only built-in module for playing wav
     import win32api
     import win32con
+    import wave     # posix-only fallback wav playback
+    import ossaudiodev as oss
 except Exception:
     pass
 
+import logging
+log = logging.getLogger('gajim.c.helpers')
+
 special_groups = (_('Transports'), _('Not in Roster'), _('Observers'), _('Groupchats'))
 
 class InvalidFormat(Exception):
@@ -75,22 +85,22 @@ def decompose_jid(jidstring):
         else:
             # host/resource
             server = jidstring[0:res_sep]
-            resource = jidstring[res_sep + 1:] or None
+            resource = jidstring[res_sep + 1:]
     else:
         if res_sep == -1:
             # user@host
-            user = jidstring[0:user_sep] or None
+            user = jidstring[0:user_sep]
             server = jidstring[user_sep + 1:]
         else:
             if user_sep < res_sep:
                 # user@host/resource
-                user = jidstring[0:user_sep] or None
+                user = jidstring[0:user_sep]
                 server = jidstring[user_sep + 1:user_sep + (res_sep - user_sep)]
-                resource = jidstring[res_sep + 1:] or None
+                resource = jidstring[res_sep + 1:]
             else:
                 # server/resource (with an @ in resource)
                 server = jidstring[0:res_sep]
-                resource = jidstring[res_sep + 1:] or None
+                resource = jidstring[res_sep + 1:]
     return user, server, resource
 
 def parse_jid(jidstring):
@@ -143,7 +153,9 @@ def prep(user, server, resource):
     """
     # This function comes from
     #http://svn.twistedmatrix.com/cvs/trunk/twisted/words/protocols/jabber/jid.py
-    if user:
+    if user is not None:
+        if len(user) < 1 or len(user) > 1023:
+            raise InvalidFormat, _('Username must be between 1 and 1023 chars')
         try:
             from xmpp.stringprepare import nodeprep
             user = nodeprep.prepare(unicode(user))
@@ -152,16 +164,20 @@ def prep(user, server, resource):
     else:
         user = None
 
-    if not server:
-        raise InvalidFormat, _('Server address required.')
-    else:
+    if server is not None:
+        if len(server) < 1 or len(server) > 1023:
+            raise InvalidFormat, _('Server must be between 1 and 1023 chars')
         try:
             from xmpp.stringprepare import nameprep
             server = nameprep.prepare(unicode(server))
         except UnicodeError:
             raise InvalidFormat, _('Invalid character in hostname.')
+    else:
+        raise InvalidFormat, _('Server address required.')
 
-    if resource:
+    if resource is not None:
+        if len(resource) < 1 or len(resource) > 1023:
+            raise InvalidFormat, _('Resource must be between 1 and 1023 chars')
         try:
             from xmpp.stringprepare import resourceprep
             resource = resourceprep.prepare(unicode(resource))
@@ -372,8 +388,18 @@ def is_in_path(command, return_abs_path=False):
             pass
     return False
 
-def exec_command(command):
-    subprocess.Popen('%s &' % command, shell=True).wait()
+def exec_command(command, use_shell=False):
+    """
+    execute a command. if use_shell is True, we run the command as is it was
+    typed in a console. So it may be dangerous if you are not sure about what
+    is executed.
+    """
+    if use_shell:
+        subprocess.Popen('%s &' % command, shell=True).wait()
+    else:
+        args = shlex.split(command.encode('utf-8'))
+        p = subprocess.Popen(args)
+        gajim.thread_interface(p.wait)
 
 def build_command(executable, parameter):
     # we add to the parameter (can hold path with spaces)
@@ -600,6 +626,9 @@ def datetime_tuple(timestamp):
 # import gajim only when needed (after decode_string is defined) see #4764
 
 import gajim
+if gajim.HAVE_PYCURL:
+    import pycurl
+    from cStringIO import StringIO
 
 def convert_bytes(string):
     suffix = ''
@@ -663,33 +692,23 @@ def get_contact_dict_for_account(account):
     return contacts_dict
 
 def launch_browser_mailer(kind, uri):
-    #kind = 'url' or 'mail'
-    if os.name == 'nt':
-        try:
-            os.startfile(uri) # if pywin32 is installed we open
-        except Exception:
-            pass
+    # kind = 'url' or 'mail'
+    if kind == 'url' and uri.startswith('file://'):
+        launch_file_manager(uri)
+        return
+    if kind in ('mail', 'sth_at_sth') and not uri.startswith('mailto:'):
+        uri = 'mailto:' + uri
 
-    else:
-        if kind in ('mail', 'sth_at_sth') and not uri.startswith('mailto:'):
-            uri = 'mailto:' + uri
+    if kind == 'url' and uri.startswith('www.'):
+        uri = 'http://' + uri
 
-        if kind == 'url' and uri.startswith('www.'):
-            uri = 'http://' + uri
-
-        if gajim.config.get('openwith') == 'gnome-open':
-            command = 'gnome-open'
-        elif gajim.config.get('openwith') == 'kfmclient exec':
-            command = 'kfmclient exec'
-        elif gajim.config.get('openwith') == 'exo-open':
-            command = 'exo-open'
-        elif gajim.config.get('openwith') == 'custom':
-            if kind == 'url':
-                command = gajim.config.get('custombrowser')
-            elif kind in ('mail', 'sth_at_sth'):
-                command = gajim.config.get('custommailapp')
-            if command == '': # if no app is configured
-                return
+    if not gajim.config.get('autodetect_browser_mailer'):
+        if kind == 'url':
+            command = gajim.config.get('custombrowser')
+        elif kind in ('mail', 'sth_at_sth'):
+            command = gajim.config.get('custommailapp')
+        if command == '': # if no app is configured
+            return
 
         command = build_command(command, uri)
         try:
@@ -697,23 +716,25 @@ def launch_browser_mailer(kind, uri):
         except Exception:
             pass
 
+    else:
+        webbrowser.open(uri)
+
+
 def launch_file_manager(path_to_open):
+    if not path_to_open.startswith('file://'):
+        uri = 'file://' + path_to_open
     if os.name == 'nt':
         try:
             os.startfile(path_to_open) # if pywin32 is installed we open
         except Exception:
             pass
     else:
-        if gajim.config.get('openwith') == 'gnome-open':
-            command = 'gnome-open'
-        elif gajim.config.get('openwith') == 'kfmclient exec':
-            command = 'kfmclient exec'
-        elif gajim.config.get('openwith') == 'exo-open':
-            command = 'exo-open'
-        elif gajim.config.get('openwith') == 'custom':
+        if not gajim.config.get('autodetect_browser_mailer'):
             command = gajim.config.get('custom_file_manager')
-        if command == '': # if no app is configured
-            return
+            if command == '': # if no app is configured
+                return
+        else:
+            command = 'xdg-open'
         command = build_command(command, path_to_open)
         try:
             exec_command(command)
@@ -785,6 +806,15 @@ def play_sound_file(path_to_soundfile):
             pass
     elif os.name == 'posix':
         if gajim.config.get('soundplayer') == '':
+            def _oss_play():
+                sndfile = wave.open(path_to_soundfile, 'rb')
+                (nc, sw, fr, nf, comptype, compname) = sndfile.getparams()
+                dev = oss.open('/dev/audio', 'w')
+                dev.setparameters(sw * 8, nc, fr)
+                dev.write(sndfile.readframes(nf))
+                sndfile.close()
+                dev.close()
+            gajim.thread_interface(_oss_play)
             return
         player = gajim.config.get('soundplayer')
         command = build_command(player, path_to_soundfile)
@@ -981,7 +1011,8 @@ def get_os_info():
                     path_to_file.endswith('arch-release'):
                         # file doesn't have version
                         text = distro_name
-                    elif path_to_file.endswith('lfs-release'): # file just has version
+                    elif path_to_file.endswith('lfs-release'):
+                        # file just has version
                         text = distro_name + ' ' + text
                 os_info = text.replace('\n', '')
                 gajim.os_info = os_info
@@ -998,8 +1029,8 @@ def get_os_info():
     return os_info
 
 
-def allow_showing_notification(account, type_ = 'notify_on_new_message',
-                advanced_notif_num = None, is_first_message = True):
+def allow_showing_notification(account, type_='notify_on_new_message',
+is_first_message=True):
     """
     Is it allowed to show nofication?
 
@@ -1007,13 +1038,6 @@ def allow_showing_notification(account, type_ = 'notify_on_new_message',
     option that need to be True e.g.: notify_on_signing is_first_message: set it
     to false when it's not the first message
     """
-    if advanced_notif_num is not None:
-        popup = gajim.config.get_per('notifications', str(advanced_notif_num),
-                'popup')
-        if popup == 'yes':
-            return True
-        if popup == 'no':
-            return False
     if type_ and (not gajim.config.get(type_) or not is_first_message):
         return False
     if gajim.config.get('autopopupaway'): # always show notification
@@ -1022,17 +1046,10 @@ def allow_showing_notification(account, type_ = 'notify_on_new_message',
         return True
     return False
 
-def allow_popup_window(account, advanced_notif_num = None):
+def allow_popup_window(account):
     """
     Is it allowed to popup windows?
     """
-    if advanced_notif_num is not None:
-        popup = gajim.config.get_per('notifications', str(advanced_notif_num),
-                'auto_open')
-        if popup == 'yes':
-            return True
-        if popup == 'no':
-            return False
     autopopup = gajim.config.get('autopopup')
     autopopupaway = gajim.config.get('autopopupaway')
     if autopopup and (autopopupaway or \
@@ -1040,14 +1057,7 @@ def allow_popup_window(account, advanced_notif_num = None):
         return True
     return False
 
-def allow_sound_notification(account, sound_event, advanced_notif_num=None):
-    if advanced_notif_num is not None:
-        sound = gajim.config.get_per('notifications', str(advanced_notif_num),
-                'sound')
-        if sound == 'yes':
-            return True
-        if sound == 'no':
-            return False
+def allow_sound_notification(account, sound_event):
     if gajim.config.get('sounddnd') or gajim.connections[account].connected != \
     gajim.SHOW_LIST.index('dnd') and gajim.config.get_per('soundevents',
     sound_event, 'enabled'):
@@ -1131,16 +1141,14 @@ def get_notification_icon_tooltip_dict():
         if total_non_messages > 0:
             if total_non_messages > max_ungrouped_events:
                 text = ngettext(
-                        '%d event pending',
-                        '%d events pending',
-                        total_non_messages, total_non_messages, total_non_messages)
+                    '%d event pending',
+                    '%d events pending',
+                    total_non_messages, total_non_messages,total_non_messages)
                 account['event_lines'].append(text)
             else:
                 for jid in non_messages.keys():
-                    text = ngettext(
-                            '%d event pending',
-                            '%d events pending',
-                            non_messages[jid], non_messages[jid], non_messages[jid])
+                    text = ngettext('%d event pending', '%d events pending',
+                        non_messages[jid], non_messages[jid], non_messages[jid])
                     text += _(' from user %s') % (jid)
                     account[account]['event_lines'].append(text)
 
@@ -1262,8 +1270,17 @@ def prepare_and_validate_gpg_keyID(account, jid, keyID):
         if jid in attached_keys and keyID:
             attachedkeyID = attached_keys[attached_keys.index(jid) + 1]
             if attachedkeyID != keyID:
-                # Mismatch! Another gpg key was expected
-                keyID += 'MISMATCH'
+                # Get signing subkeys for the attached key
+                subkeys = []
+                for key in gajim.connections[account].gpg.list_keys():
+                    if key['keyid'][8:] == attachedkeyID:
+                        subkeys = [subkey[0][8:] for subkey in key['subkeys'] \
+                            if subkey[1] == 's']
+                        break
+
+                if keyID not in subkeys:
+                    # Mismatch! Another gpg key was expected
+                    keyID += 'MISMATCH'
         elif jid in attached_keys:
             # An unsigned presence, just use the assigned key
             keyID = attached_keys[attached_keys.index(jid) + 1]
@@ -1273,9 +1290,11 @@ def prepare_and_validate_gpg_keyID(account, jid, keyID):
             if keyID in public_keys:
                 for u in gajim.contacts.get_contacts(account, jid):
                     u.keyID = keyID
-                keys_str = gajim.config.get_per('accounts', account, 'attached_gpg_keys')
+                keys_str = gajim.config.get_per('accounts', account,
+                    'attached_gpg_keys')
                 keys_str += jid + ' ' + keyID + ' '
-                gajim.config.set_per('accounts', account, 'attached_gpg_keys', keys_str)
+                gajim.config.set_per('accounts', account, 'attached_gpg_keys',
+                    keys_str)
         elif keyID is None:
             keyID = 'UNKNOWN'
     return keyID
@@ -1311,7 +1330,7 @@ def update_optional_features(account = None):
             gajim.gajim_optional_features[a].append(xmpp.NS_ESESSION)
         if gajim.config.get_per('accounts', a, 'answer_receipts'):
             gajim.gajim_optional_features[a].append(xmpp.NS_RECEIPTS)
-        if gajim.HAVE_FARSIGHT:
+        if gajim.HAVE_FARSTREAM:
             gajim.gajim_optional_features[a].append(xmpp.NS_JINGLE)
             gajim.gajim_optional_features[a].append(xmpp.NS_JINGLE_RTP)
             gajim.gajim_optional_features[a].append(xmpp.NS_JINGLE_RTP_AUDIO)
@@ -1356,3 +1375,173 @@ def get_subscription_request_msg(account=None):
             name = nick
         s = Template(s).safe_substitute({'name': name})
         return s
+
+def replace_dataform_media(form, stanza):
+    import xmpp
+    found = False
+    for field in form.getTags('field'):
+        for media in field.getTags('media'):
+            for uri in media.getTags('uri'):
+                uri_data = uri.getData()
+                if uri_data.startswith('cid:'):
+                    uri_data = uri_data[4:]
+                    for data in stanza.getTags('data', namespace=xmpp.NS_BOB):
+                        if data.getAttr('cid') == uri_data:
+                            uri.setData(data.getData())
+                            found = True
+    return found
+
+def get_proxy_info(account):
+    p = gajim.config.get_per('accounts', account, 'proxy')
+    if not p:
+        if gajim.config.get_per('accounts', account, 'use_env_http_proxy'):
+            try:
+                try:
+                    env_http_proxy = os.environ['HTTP_PROXY']
+                except Exception:
+                    env_http_proxy = os.environ['http_proxy']
+                env_http_proxy = env_http_proxy.strip('"')
+                # Dispose of the http:// prefix
+                env_http_proxy = env_http_proxy.split('://')[-1]
+                env_http_proxy = env_http_proxy.split('@')
+
+                if len(env_http_proxy) == 2:
+                    login = env_http_proxy[0].split(':')
+                    addr = env_http_proxy[1].split(':')
+                else:
+                    login = ['', '']
+                    addr = env_http_proxy[0].split(':')
+
+                proxy = {'host': addr[0], 'type' : u'http', 'user':login[0]}
+
+                if len(addr) == 2:
+                    proxy['port'] = addr[1]
+                else:
+                    proxy['port'] = 3128
+
+                if len(login) == 2:
+                    proxy['pass'] = login[1]
+                    proxy['useauth'] = True
+                else:
+                    proxy['pass'] = u''
+                return proxy
+
+            except Exception:
+                proxy = None
+        p = gajim.config.get('global_proxy')
+    if p and p in gajim.config.get_per('proxies'):
+        proxy = {}
+        proxyptr = gajim.config.get_per('proxies', p)
+        if not proxyptr:
+            return proxy
+        for key in proxyptr.keys():
+            proxy[key] = proxyptr[key][1]
+        return proxy
+
+def _get_img_direct(attrs):
+    """
+    Download an image. This function should be launched in a separated thread.
+    """
+    mem, alt = '', ''
+    # Wait maximum 5s for connection
+    socket.setdefaulttimeout(5)
+    try:
+        req = urllib2.Request(attrs['src'])
+        req.add_header('User-Agent', 'Gajim ' + gajim.version)
+        f = urllib2.urlopen(req)
+    except Exception, ex:
+        log.debug('Error loading image %s ' % attrs['src']  + str(ex))
+        pixbuf = None
+        alt = attrs.get('alt', 'Broken image')
+    else:
+        # Wait 0.5s between each byte
+        try:
+            f.fp._sock.fp._sock.settimeout(0.5)
+        except Exception:
+            pass
+        # Max image size = 2 MB (to try to prevent DoS)
+        deadline = time.time() + 3
+        while True:
+            if time.time() > deadline:
+                log.debug('Timeout loading image %s ' % attrs['src'])
+                mem = ''
+                alt = attrs.get('alt', '')
+                if alt:
+                    alt += '\n'
+                alt += _('Timeout loading image')
+                break
+            try:
+                temp = f.read(100)
+            except socket.timeout, ex:
+                log.debug('Timeout loading image %s ' % attrs['src'] + str(ex))
+                alt = attrs.get('alt', '')
+                if alt:
+                    alt += '\n'
+                alt += _('Timeout loading image')
+                break
+            if temp:
+                mem += temp
+            else:
+                break
+            if len(mem) > 2*1024*1024:
+                alt = attrs.get('alt', '')
+                if alt:
+                    alt += '\n'
+                alt += _('Image is too big')
+                break
+    return (mem, alt)
+
+def _get_img_proxy(attrs, proxy):
+    """
+    Download an image through a proxy. This function should be launched in a
+    separated thread.
+    """
+    if not gajim.HAVE_PYCURL:
+        return '', _('PyCURL is not installed')
+    mem, alt = '', ''
+    try:
+        b = StringIO()
+        c = pycurl.Curl()
+        c.setopt(pycurl.URL, attrs['src'].encode('utf-8'))
+        c.setopt(pycurl.FOLLOWLOCATION, 1)
+        c.setopt(pycurl.CONNECTTIMEOUT, 5)
+        c.setopt(pycurl.TIMEOUT, 10)
+        c.setopt(pycurl.MAXFILESIZE, 2000000)
+        c.setopt(pycurl.WRITEFUNCTION, b.write)
+        c.setopt(pycurl.USERAGENT, 'Gajim ' + gajim.version)
+        # set proxy
+        c.setopt(pycurl.PROXY, proxy['host'].encode('utf-8'))
+        c.setopt(pycurl.PROXYPORT, proxy['port'])
+        if proxy['useauth']:
+            c.setopt(pycurl.PROXYUSERPWD, proxy['user'].encode('utf-8')\
+                + ':' + proxy['pass'].encode('utf-8'))
+            c.setopt(pycurl.PROXYAUTH, pycurl.HTTPAUTH_ANY)
+        if proxy['type'] == 'http':
+            c.setopt(pycurl.PROXYTYPE, pycurl.PROXYTYPE_HTTP)
+        elif proxy['type'] == 'socks5':
+            c.setopt(pycurl.PROXYTYPE, pycurl.PROXYTYPE_SOCKS5)
+        x = c.perform()
+        c.close()
+        t = b.getvalue()
+        return (t, attrs.get('alt', ''))
+    except pycurl.error, ex:
+        alt = attrs.get('alt', '')
+        if alt:
+            alt += '\n'
+        if ex[0] == pycurl.E_FILESIZE_EXCEEDED:
+            alt += _('Image is too big')
+        elif ex[0] == pycurl.E_OPERATION_TIMEOUTED:
+            alt += _('Timeout loading image')
+        else:
+            alt += _('Error loading image')
+    except Exception, ex:
+        log.debug('Error loading image %s ' % attrs['src']  + str(ex))
+        pixbuf = None
+        alt = attrs.get('alt', 'Broken image')
+    return ('', alt)
+
+def download_image(account, attrs):
+    proxy = get_proxy_info(account)
+    if proxy and proxy['type'] in ('http', 'socks5'):
+        return _get_img_proxy(attrs, proxy)
+    return _get_img_direct(attrs)
