@@ -59,7 +59,7 @@ class JingleRTPContent(JingleContent):
         self.callbacks['session-terminate'] += [self.__stop]
         self.callbacks['session-terminate-sent'] += [self.__stop]
 
-    def setup_stream(self):
+    def setup_stream(self, on_src_pad_added):
         # pipeline and bus
         self.pipeline = gst.Pipeline()
         bus = self.pipeline.get_bus()
@@ -68,13 +68,12 @@ class JingleRTPContent(JingleContent):
 
         # conference
         self.conference = gst.element_factory_make('fsrtpconference')
-        self.conference.set_property('sdes-cname', self.session.ourjid)
         self.pipeline.add(self.conference)
         self.funnel = None
 
         self.p2psession = self.conference.new_session(self.farstream_media)
 
-        participant = self.conference.new_participant(self.session.peerjid)
+        participant = self.conference.new_participant()
         # FIXME: Consider a workaround, here...
         # pidgin and telepathy-gabble don't follow the XEP, and it won't work
         # due to bad controlling-mode
@@ -93,7 +92,9 @@ class JingleRTPContent(JingleContent):
                     params['stun-ip'] = ip
 
         self.p2pstream = self.p2psession.new_stream(participant,
-                farstream.DIRECTION_RECV, 'nice', params)
+                farstream.DIRECTION_RECV)
+        self.p2pstream.connect('src-pad-added', on_src_pad_added)
+        self.p2pstream.set_transmitter('nice', params)
 
     def is_ready(self):
         return (JingleContent.is_ready(self) and self.candidates_ready)
@@ -117,7 +118,7 @@ class JingleRTPContent(JingleContent):
         # FIXME: connectivity should not be etablished yet
         # Instead, it should be etablished after session-accept!
         if self.sent:
-            self.p2pstream.set_remote_candidates(candidates)
+            self.p2pstream.add_remote_candidates(candidates)
 
     def batch_dtmf(self, events):
         """
@@ -144,11 +145,10 @@ class JingleRTPContent(JingleContent):
                     '#': farstream.DTMF_EVENT_POUND}[event]
         else:
             event = int(event)
-        self.p2psession.start_telephony_event(event, 2,
-                farstream.DTMF_METHOD_RTP_RFC4733)
+        self.p2psession.start_telephony_event(event, 2)
 
     def _stop_dtmf(self):
-        self.p2psession.stop_telephony_event(farstream.DTMF_METHOD_RTP_RFC4733)
+        self.p2psession.stop_telephony_event()
 
     def _fill_content(self, content):
         content.addChild(xmpp.NS_JINGLE_RTP + ' description',
@@ -175,8 +175,17 @@ class JingleRTPContent(JingleContent):
             elif name == 'farstream-recv-codecs-changed':
                 pass
             elif name == 'farstream-codecs-changed':
-                if self.sent and self.p2psession.get_property('codecs-ready'):
+                if self.sent and self.p2psession.get_property('codecs'):
                     self.send_description_info()
+                    if self.transport.remote_candidates:
+                        # those lines MUST be done after we get info on our
+                        # codecs
+                        self.p2pstream.add_remote_candidates(
+                            self.transport.remote_candidates)
+                        self.transport.remote_candidates = []
+                        self.p2pstream.set_property('direction',
+                            farstream.DIRECTION_BOTH)
+
             elif name == 'farstream-local-candidates-prepared':
                 self.candidates_ready = True
                 if self.is_ready():
@@ -194,10 +203,9 @@ class JingleRTPContent(JingleContent):
                     reason.setTag('failed-transport')
                     self.session.remove_content(self.creator, self.name, reason)
             elif name == 'farstream-error':
-                log.error('Farsight error #%d!\nMessage: %s\nDebug: %s' % (
+                log.error('Farstream error #%d!\nMessage: %s' % (
                     message.structure['error-no'],
-                    message.structure['error-msg'],
-                    message.structure['debug-msg']))
+                    message.structure['error-msg']))
         elif message.type == gst.MESSAGE_ERROR:
             # TODO: Fix it to fallback to videotestsrc anytime an error occur,
             # or raise an error, Jingle way
@@ -235,11 +243,15 @@ class JingleRTPContent(JingleContent):
 
     def on_negotiated(self):
         if self.accepted:
-            if self.transport.remote_candidates:
-                self.p2pstream.set_remote_candidates(self.transport.remote_candidates)
-                self.transport.remote_candidates = []
-            # TODO: farstream.DIRECTION_BOTH only if senders='both'
-            self.p2pstream.set_property('direction', farstream.DIRECTION_BOTH)
+            if self.p2psession.get_property('codecs'):
+                # those lines MUST be done after we get info on our codecs
+                if self.transport.remote_candidates:
+                    self.p2pstream.add_remote_candidates(
+                        self.transport.remote_candidates)
+                    self.transport.remote_candidates = []
+                # TODO: farstream.DIRECTION_BOTH only if senders='both'
+                self.p2pstream.set_property('direction',
+                    farstream.DIRECTION_BOTH)
         JingleContent.on_negotiated(self)
 
     def __on_remote_codecs(self, stanza, content, error, action):
@@ -318,7 +330,7 @@ class JingleAudio(JingleRTPContent):
         self.out_volume.set_property('volume', vol)
 
     def setup_stream(self):
-        JingleRTPContent.setup_stream(self)
+        JingleRTPContent.setup_stream(self, self._on_src_pad_added)
 
         # Configure SPEEX
         # Workaround for psi (not needed since rev
@@ -348,7 +360,6 @@ class JingleAudio(JingleRTPContent):
 
         self.src_bin.get_pad('src').link(self.p2psession.get_property(
                 'sink-pad'))
-        self.p2pstream.connect('src-pad-added', self._on_src_pad_added)
 
         # The following is needed for farstream to process ICE requests:
         self.pipeline.set_state(gst.STATE_PLAYING)
@@ -363,7 +374,7 @@ class JingleVideo(JingleRTPContent):
         # TODO: Everything is not working properly:
         # sometimes, one window won't show up,
         # sometimes it'll freeze...
-        JingleRTPContent.setup_stream(self)
+        JingleRTPContent.setup_stream(self, self._on_src_pad_added)
 
         # the local parts
         if gajim.config.get('video_framerate'):
@@ -395,7 +406,6 @@ class JingleVideo(JingleRTPContent):
 
         self.src_bin.get_pad('src').link(self.p2psession.get_property(
             'sink-pad'))
-        self.p2pstream.connect('src-pad-added', self._on_src_pad_added)
 
         # The following is needed for farstream to process ICE requests:
         self.pipeline.set_state(gst.STATE_PLAYING)
