@@ -388,20 +388,25 @@ class IdleQueue:
             self.unplug_idle(fd)
             return False
 
+        read_write = False
         if flags & PENDING_READ:
             #print 'waiting read on %d, flags are %d' % (fd, flags)
             obj.pollin()
-            return True
+            read_write = True
 
-        elif flags & PENDING_WRITE:
+        elif flags & PENDING_WRITE and not flags & IS_CLOSED:
             obj.pollout()
-            return True
+            read_write = True
 
-        elif flags & IS_CLOSED:
+        if flags & IS_CLOSED:
             # io error, don't expect more events
             self.remove_timeout(obj.fd)
             self.unplug_idle(obj.fd)
             obj.pollend()
+            return False
+
+        if read_write:
+            return True
         return False
 
     def process(self):
