@@ -3,7 +3,7 @@
 ##
 ## Copyright (C) 2006 Nikos Kouremenos <kourem AT gmail.com>
 ## Copyright (C) 2006-2007 Jean-Marie Traissard <jim AT lapin.org>
-## Copyright (C) 2006-2010 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2006-2012 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2007 Lukas Petrovicky <lukas AT petrovicky.net>
 ##                    Julien Pivotto <roidelapluie AT gmail.com>
 ## Copyright (C) 2008 Jonathan Schleifer <js-gajim AT webkeks.org>
@@ -246,7 +246,7 @@ class StatusIcon:
         item.connect('activate', self.on_show_menuitem_activate, 'offline')
 
         iskey = connected_accounts > 0 and not (connected_accounts == 1 and
-                        gajim.connections[gajim.connections.keys()[0]].is_zeroconf)
+            gajim.zeroconf_is_connected())
         chat_with_menuitem.set_sensitive(iskey)
         single_message_menuitem.set_sensitive(iskey)
         join_gc_menuitem.set_sensitive(iskey)
@@ -367,7 +367,6 @@ class StatusIcon:
 
     def on_sounds_mute_menuitem_activate(self, widget):
         gajim.config.set('sounds_on', not widget.get_active())
-        gajim.interface.save_config()
 
     def on_show_roster_menuitem_activate(self, widget):
         win = gajim.interface.roster.window
@@ -396,7 +395,8 @@ class StatusIcon:
 
                 # we could be in another VD right now. eg vd2
                 # and we want to show it in vd2
-                if not gtkgui_helpers.possibly_move_window_in_current_desktop(win):
+                if not gtkgui_helpers.possibly_move_window_in_current_desktop(
+                win) and gajim.config.get('save-roster-position'):
                     x, y = win.get_position()
                     gajim.config.set('roster_x-position', x)
                     gajim.config.set('roster_y-position', y)
@@ -404,9 +404,10 @@ class StatusIcon:
             else:
                 if not win.get_property('visible'):
                     win.show_all()
-                    gtkgui_helpers.move_window(win,
-                        gajim.config.get('roster_x-position'),
-                        gajim.config.get('roster_y-position'))
+                    if gajim.config.get('save-roster-position'):
+                        gtkgui_helpers.move_window(win,
+                            gajim.config.get('roster_x-position'),
+                            gajim.config.get('roster_y-position'))
                 if not gajim.config.get('roster_window_skip_taskbar'):
                     win.set_property('skip-taskbar-hint', False)
                 win.present_with_time(gtk.get_current_event_time())
@@ -417,6 +418,12 @@ class StatusIcon:
         account, jid, event = gajim.events.get_first_systray_event()
         if not event:
             return
+        win = gajim.interface.roster.window
+        if not win.get_property('visible') and gajim.config.get(
+        'save-roster-position'):
+            gtkgui_helpers.move_window(win,
+                gajim.config.get('roster_x-position'),
+                gajim.config.get('roster_y-position'))
         gajim.interface.handle_event(account, jid, event.type_)
 
     def on_middle_click(self):
